@@ -1,7 +1,7 @@
 
 //  RQuantLib -- R interface to the QuantLib libraries
 //
-//  Copyright (C) 2002 - 2021  Dirk Eddelbuettel
+//  Copyright (C) 2002 - 2022  Dirk Eddelbuettel
 //  Copyright (C) 2009 - 2012  Khanh Nguyen and Dirk Eddelbuettel
 //
 //  This file is part of RQuantLib.
@@ -553,8 +553,12 @@ Rcpp::List convertibleZeroBondEngine(Rcpp::List rparam,
         QuantLib::ext::shared_ptr<QuantLib::BlackScholesMertonProcess>(new QuantLib::BlackScholesMertonProcess(underlying, dividendYield,
                                                                                                        rff, volatility));
 
+#if QL_HEX_VERSION < 0x012500f0
     QuantLib::RelinkableHandle<QuantLib::Quote> creditSpread;
     creditSpread.linkTo(QuantLib::ext::shared_ptr<QuantLib::Quote>(new QuantLib::SimpleQuote(creditSpreadQuote)));
+#else
+    QuantLib::Handle<QuantLib::Quote> creditSpread(QuantLib::ext::shared_ptr<QuantLib::Quote>(new QuantLib::SimpleQuote(creditSpreadQuote)));
+#endif
 
     QuantLib::ext::shared_ptr<QuantLib::Exercise> euExercise(new QuantLib::EuropeanExercise(maturityDate));
     QuantLib::ext::shared_ptr<QuantLib::Exercise> amExercise(new QuantLib::AmericanExercise(issueDate, maturityDate));
@@ -562,7 +566,11 @@ Rcpp::List convertibleZeroBondEngine(Rcpp::List rparam,
 
     QuantLib::Size timeSteps = 1001;
     QuantLib::ext::shared_ptr<QuantLib::PricingEngine>
-        engine(new QuantLib::BinomialConvertibleEngine<QuantLib::CoxRossRubinstein>(blackProcess, timeSteps));
+        engine(new QuantLib::BinomialConvertibleEngine<QuantLib::CoxRossRubinstein>(blackProcess, timeSteps
+#if QL_HEX_VERSION >= 0x012500f0
+                                                                                    , creditSpread, dividendSchedule
+#endif
+                                                                                    ));
 
     QuantLib::Handle<QuantLib::YieldTermStructure>
         discountCurve(QuantLib::ext::shared_ptr<QuantLib::YieldTermStructure>(new QuantLib::ForwardSpreadedTermStructure(rff, creditSpread)));
@@ -570,8 +578,13 @@ Rcpp::List convertibleZeroBondEngine(Rcpp::List rparam,
     QuantLib::Schedule sch(issueDate, maturityDate, QuantLib::Period(freq), calendar,
                            bdc, bdc, QuantLib::DateGeneration::Backward, false);
     QuantLib::ConvertibleZeroCouponBond bond(ex, conversionRatio,
-                                             dividendSchedule, callabilitySchedule,
+#if QL_HEX_VERSION < 0x012500f0
+                                             dividendSchedule,
+#endif
+                                             callabilitySchedule,
+#if QL_HEX_VERSION < 0x012500f0
                                              creditSpread,
+#endif
                                              issueDate, settlementDays,
                                              dc, sch,
                                              redemption);
@@ -660,7 +673,11 @@ Rcpp::List convertibleFixedBondEngine(Rcpp::List rparam,
 
     QuantLib::Size timeSteps = 1001;
     QuantLib::ext::shared_ptr<QuantLib::PricingEngine>
-        engine(new QuantLib::BinomialConvertibleEngine<QuantLib::CoxRossRubinstein>(blackProcess, timeSteps));
+        engine(new QuantLib::BinomialConvertibleEngine<QuantLib::CoxRossRubinstein>(blackProcess, timeSteps
+#if QL_HEX_VERSION >= 0x012500f0
+                                                                                    , creditSpread, dividendSchedule
+#endif
+                                                                                    ));
 
     QuantLib::Handle<QuantLib::YieldTermStructure>
         discountCurve(QuantLib::ext::shared_ptr<QuantLib::YieldTermStructure>(new QuantLib::ForwardSpreadedTermStructure(rff,
@@ -671,8 +688,14 @@ Rcpp::List convertibleFixedBondEngine(Rcpp::List rparam,
                            bdc, bdc,
                            QuantLib::DateGeneration::Backward, false);
     QuantLib::ConvertibleFixedCouponBond bond(ex, conversionRatio,
-                                              dividendSchedule, callabilitySchedule,
-                                              creditSpread,issueDate,
+#if QL_HEX_VERSION < 0x012500f0
+                                              dividendSchedule,
+#endif
+                                              callabilitySchedule,
+#if QL_HEX_VERSION < 0x012500f0
+                                              creditSpread,
+#endif
+                                              issueDate,
                                               settlementDays,
                                               Rcpp::as<std::vector <double> >(rates),
                                               dc, sch, redemption);
@@ -781,7 +804,11 @@ Rcpp::List convertibleFloatingBondEngine(Rcpp::List rparam,
 
 
     QuantLib::Size timeSteps = 1001;
-    QuantLib::ext::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::BinomialConvertibleEngine<QuantLib::CoxRossRubinstein>(blackProcess, timeSteps));
+    QuantLib::ext::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::BinomialConvertibleEngine<QuantLib::CoxRossRubinstein>(blackProcess, timeSteps
+#if QL_HEX_VERSION >= 0x012500f0
+                                                                                                                                   , creditSpread, dividendSchedule
+#endif
+                                                                                                                                   ));
 
     QuantLib::Handle<QuantLib::YieldTermStructure>
         discountCurve(QuantLib::ext::shared_ptr<QuantLib::YieldTermStructure>(new QuantLib::ForwardSpreadedTermStructure(rff,
@@ -789,9 +816,15 @@ Rcpp::List convertibleFloatingBondEngine(Rcpp::List rparam,
     QuantLib::Natural fixingDays = 2;
     QuantLib::Schedule sch(issueDate, maturityDate, QuantLib::Period(freq),
                            calendar, bdc, bdc, QuantLib::DateGeneration::Backward, false);
-    QuantLib::ConvertibleFloatingRateBond bond(ex, conversionRatio, dividendSchedule, callabilitySchedule,
-                                               creditSpread,issueDate,
-                                               settlementDays,iborindex,fixingDays, spreads,
+    QuantLib::ConvertibleFloatingRateBond bond(ex, conversionRatio,
+#if QL_HEX_VERSION < 0x012500f0
+                                               dividendSchedule,
+#endif
+                                               callabilitySchedule,
+#if QL_HEX_VERSION < 0x012500f0
+                                               creditSpread,
+#endif
+                                               issueDate, settlementDays,iborindex,fixingDays, spreads,
                                                dc, sch, redemption);
     bond.setPricingEngine(engine);
 
@@ -880,120 +913,6 @@ Rcpp::List callableBondEngine(Rcpp::List rparam,
     return R_NilValue;
 }
 
-// RcppExport SEXP CMSBond(SEXP bondparams, SEXP iborIndex, SEXP swapIndexParam,
-//                            SEXP capsVec, SEXP floorsVec, SEXP gearingsVec,
-//                            SEXP spreadsVec, SEXP swaptionVolSEXP, SEXP atmOptionTenorsSEXP,
-//                            SEXP atmSwapTenorsSEXP, SEXP volMatrixSEXP, SEXP pricer,
-//                            SEXP iborIndexDate, SEXP iborIndexRates)
-// {
-//     SEXP rl=R_NilValue;
-//     char* exceptionMesg=NULL;
-//     try {
-
-//         std::vector<double> gearings = getDoubleVector(gearingsVec);
-//         std::vector<double> spreads = getDoubleVector(spreadsVec);
-//         std::vector<double> caps = getDoubleVector(capsVec);
-//         std::vector<double> floors = getDoubleVector(floorsVec);
-
-//         RcppStringVector strVec(atmOptionTenorsSEXP);
-//         std::vector<string> atmOptionTenors(strVec.stlVector());
-//         RcppStringVector strVec2(atmSwapTenorsSEXP);
-//         std::vector<string> atmSwapTenors(strVec2.stlVector());
-
-//         RcppMatrix<double> m(volMatrixSEXP);
-
-//         QuantLib::Handle<QuantLib::YieldTermStructure> termStructure(rebuildCurveFromZeroRates(
-//                                                                            iborIndexDate,iborIndexRates));
-//         Rcppparams iborparams(iborIndex);
-//         std::string ibortype = iborparams.getStringValue("type");
-//         std::string iborlength = iborparams.getStringValue("length");
-//         QuantLib::ext::shared_ptr<QuantLib::IborIndex> ibor = getIborIndex(termStructure, ibortype,
-//                                                          iborlength);
-//         //fix tenor to make sure it is converted by matchparam
-//         Rcppparams swapparams(swapIndexParam);
-//         std::string familyName = swapparams.getStringValue("familyName");
-//         std::double tenor = swapparams.getDoubleValue("tenor");
-//         std::double settlementDays = swapparams.getDoubleValue("settlementDays");
-//         std::string currency = swapparams.getStringValue("currency");
-//         std::string fixedLegTenor = swapparams.getDoubleValue("fixedLegTenor");
-//         std::string fixedLegConvention = swapparams.getDoubleValue("fixedLegConvention");
-//         std::string fixedLegDayCounter = swapparams.getDoubleValue("fixedLegDayCounter");
-//         std::string cal = swapparams.getStringValue("calendar");
-//         QuantLib::Calendar calendar;
-//         if(!cal.empty()) {
-//             QuantLib::ext::shared_ptr<QuantLib::Calendar> p = getCalendar(cal);
-//             calendar = *p;
-//         }
-//         QuantLib::BusinessDayConvention fixedLegBDC = getBusinessDayConvention(fixedLegConvention);
-//         QuantLib::DayCounter fixedLedDC = getDayCounter(fixedLegDayCounter);
-
-//         QuantLib::ext::shared_ptr<QuantLib::SwapIndex> swapIndex(new QuantLib::SwapIndex(familiName,
-//                                                              getPeriodFromString(fixedLegTenor),
-//                                                              settlemenDays,
-//                                                              currency,
-//                                                              calendar,
-//                                                              getPeriodFromString(fixedLegTenor),
-//                                                              fixedLegBDC,
-//                                                              fixedLegDC,
-//                                                              ibor));
-
-//         Rcppparams pricerparams(pricer);
-//         std::string pricerType = pricerparams.getStringValue("type");
-//         std::double zeroMeanRev = pricerparams.getDoubleValue("zeroMeanRev");
-
-//         Rcppparams swaptionVolParams(swaptionVolSEXP);
-//         std::string swaptionCal = swaptionVolParams.getStringValue("calendar");
-//         std::double swaptionBDC = swaptionVolParams.getDoubleValue("businessDayConvention");
-//         std::double swaptionDC = swaptionVolParams.getDoubleValue("dayCounter");
-//         QuantLib::Handle<QuantLib::SwaptionVolatilityStructure> atmVol;
-//         atmVol = QuantLib::Handle<QuantLib::SwaptionVolatilityStructure>(
-//                                                      QuantLib::ext::shared_ptr<QuantLib::SwaptionVolatilityStructure>
-//                                                      new QuantLib::SwaptionVolatilityMatrix(swapCal,
-//                                                                                   swaptionBDC,
-//                                                                                   atmOptionTenors,
-//                                                                                   atmSwapTenors,
-//                                                                                   m,
-//                                                                                   swaptionDC));
-
-
-//         QuantLib::ext::shared_ptr<QuantLib::CmsCouponPricer> pricer(new QuantLib::NumericHaganPricer(atmVol, yieldCurveModel,
-//                                                                          zeroMeanRev));
-
-
-//         Rcppparams rparams(bondparams);
-//         Rcpp::Date mDate = Rcpp::Date(Rcpp::as<int>(rparam["maturityDate"]));
-//         Rcpp::Date iDate = Rcpp::Date(Rcpp::as<int>(rparam["issueDate"]));
-//         Rcpp::Date pDate = Rcpp::Date(Rcpp::as<int>(rparam["paymentDate"]));
-//         QuantLib::Date maturityDate(dateFromR(mDate));
-//         QuantLib::Date issueDate(dateFromR(iDate));
-//         QuantLib::Date paymentDate(dateFromR(pDate));
-//         std::double nomial = rparams.getDoubleValue("nomial");
-
-//         CappedFlooredCmsCoupon coupon(paymentDate, nomial,
-//                                       issueDate, maturityDate,
-//                                       settlementDays, swapIndex,
-//                                       gearings, spreads,
-//                                       caps, floors,
-//                                       issueDate, maturityDate,
-//                                       dayCounter);
-
-//         pricer->setSwaptionVolatility(atmVol);
-//         coupon.setPricer(pricer);
-
-
-
-//     } catch(std::exception& ex) {
-//         exceptionMesg = copyMessageToR(ex.what());
-//     } catch(...) {
-//         exceptionMesg = copyMessageToR("unknown reason");
-//     }
-
-//     if(exceptionMesg != NULL)
-//         Rf_error(exceptionMesg);
-
-//     return rl;
-// }
-
 // [[Rcpp::export]]
 Rcpp::List fittedBondCurveEngine(Rcpp::List curveparam,
                                  Rcpp::NumericVector length,
